@@ -1,4 +1,4 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
@@ -31,16 +31,19 @@ from .bloch import (
     display_bloch_sphere_from_density_matrices,
     display_bloch_spheres_and_correlations,
 )
+from .confidence_ellipses import plot_confidence_ellipses
 from .controls import (
     plot_controls,
     plot_sequences,
     plot_smooth_controls,
 )
+from .cost_history import plot_cost_history
 from .discriminators import (
     plot_discriminator,
     plot_xdata,
 )
 from .filter_functions import plot_filter_functions
+from .populations import plot_populations
 from .style import (
     QCTRL_STYLE_COLORS,
     get_qctrl_style,
@@ -53,12 +56,15 @@ __all__ = [
     "display_bloch_sphere_from_density_matrices",
     "display_bloch_spheres_and_correlations",
     "get_qctrl_style",
+    "plot_confidence_ellipses",
     "plot_controls",
+    "plot_cost_history",
     "plot_discriminator",
     "plot_filter_functions",
+    "plot_populations",
     "plot_sequences",
     "plot_smooth_controls",
     "plot_xdata",
 ]
 
-__version__ = "2.12.2"
+__version__ = "3.2.1"
