@@ -1,4 +1,4 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
@@ -22,9 +22,15 @@ from typing import (
 )
 
 import numpy as np
-from matplotlib.ticker import ScalarFormatter
 
-from .style import qctrl_style
+from .style import (
+    FIG_WIDTH,
+    qctrl_style,
+)
+from .utils import (
+    create_axes,
+    get_units,
+)
 
 
 @qctrl_style()
@@ -129,17 +135,18 @@ def plot_controls(
             durations = control["durations"]
             values = control["values"]
         # Create plot data for the control.
-        values = np.array(values)
         plots_data = plots_data + _create_plots_data_from_control(
-            name, durations, values, polar, unit_symbol, two_pi_factor
+            name, durations, np.array(values), polar, unit_symbol, two_pi_factor
         )
 
     # Get the appropriate scaling for the time axis based on the total durations of all controls.
-    time_scaling, time_prefix = _get_units(
+    time_scaling, time_prefix = get_units(
         [sum(plot_data.xdata) for plot_data in plots_data]
     )
 
-    axes_list = _create_axes(figure, len(plots_data), width=10.0, height=2.5)
+    axes_list = create_axes(
+        figure, 1, len(plots_data), width=FIG_WIDTH, height=FIG_WIDTH / 4, share_x=True
+    ).flatten()
 
     if smooth:
         for axes, plot_data in zip(axes_list, plots_data):
@@ -158,11 +165,11 @@ def plot_controls(
                 / time_scaling
             )
             # Pad each values array with leading and trailing zeros, to indicate that the pulse is
-            # zero outside the plot domain. Length of 'values' is m+2.
-            values = np.pad(plot_data.values, ((1, 1)), "constant")
+            # zero outside the plot domain. Length of 'values_arr' is m+2.
+            values_arr = np.pad(plot_data.values, ((1, 1)), "constant")
 
-            axes.plot(times, values, linewidth=2)
-            axes.fill(times, values, alpha=0.3)
+            axes.plot(times, values_arr, linewidth=2)
+            axes.fill(times, values_arr, alpha=0.3)
 
             axes.axhline(y=0, linewidth=1, zorder=-1)
             axes.set_ylabel(plot_data.ylabel)
@@ -173,8 +180,8 @@ def plot_controls(
             # is m+1 (m is the number of segments).
             times = np.insert(np.cumsum(plot_data.xdata), 0, 0.0) / time_scaling
             # Pad each values array with leading and trailing zeros, to indicate that the pulse is
-            # zero outside the plot domain. Length of 'values' is m+2.
-            values = np.pad(plot_data.values, ((1, 1)), "constant")
+            # zero outside the plot domain. Length of 'values_arr' is m+2.
+            values_arr = np.pad(plot_data.values, ((1, 1)), "constant")
 
             #              *---v2--*
             #              |       |
@@ -187,10 +194,10 @@ def plot_controls(
             # [0, 0, 1, 1, ..., m-1, m-1, m, m  ], and from the values arrays at indices
             # [0, 1, 1, 2, ..., m-1, m,   m, m+1].
             time_indices = np.repeat(np.arange(len(times)), 2)
-            value_indices = np.repeat(np.arange(len(values)), 2)[1:-1]
+            value_indices = np.repeat(np.arange(len(values_arr)), 2)[1:-1]
 
-            axes.plot(times[time_indices], values[value_indices], linewidth=2)
-            axes.fill(times[time_indices], values[value_indices], alpha=0.3)
+            axes.plot(times[time_indices], values_arr[value_indices], linewidth=2)
+            axes.fill(times[time_indices], values_arr[value_indices], alpha=0.3)
 
             axes.axhline(y=0, linewidth=1, zorder=-1)
             for time in times:
@@ -269,26 +276,27 @@ def plot_smooth_controls(
             times = control["times"]
             values = control["values"]
         # Create plot data for the control.
-        values = np.array(values)
         plots_data = plots_data + _create_plots_data_from_control(
-            name, times, values, polar, unit_symbol, two_pi_factor
+            name, times, np.array(values), polar, unit_symbol, two_pi_factor
         )
 
     # Get the appropriate scaling for the time axis based on the total durations of all controls.
-    time_scaling, time_prefix = _get_units(
+    time_scaling, time_prefix = get_units(
         [max(plot_data.xdata) for plot_data in plots_data]
     )
 
-    axes_list = _create_axes(figure, len(plots_data), width=10.0, height=2.5)
+    axes_list = create_axes(
+        figure, 1, len(plots_data), width=FIG_WIDTH, height=FIG_WIDTH / 4, share_x=True
+    ).flatten()
 
     for axes, plot_data in zip(axes_list, plots_data):
         # Pad with leading and trailing zeros, to indicate that the pulse is zero outside the plot
         # domain.
         times = np.pad(plot_data.xdata, ((1, 1)), "edge") / time_scaling
-        values = np.pad(plot_data.values, ((1, 1)), "constant")
+        values_arr = np.pad(plot_data.values, ((1, 1)), "constant")
 
-        axes.plot(times, values, linewidth=2)
-        axes.fill(times, values, alpha=0.3)
+        axes.plot(times, values_arr, linewidth=2)
+        axes.fill(times, values_arr, alpha=0.3)
 
         axes.axhline(y=0, linewidth=1, zorder=-1)
 
@@ -303,47 +311,6 @@ def plot_smooth_controls(
 _PlotData = namedtuple("_PlotData", ["ylabel", "xdata", "values"])
 
 
-def _get_units(values):
-    """
-    For the given range of values, calculates the units to be used for plotting.  Specifically,
-    returns a tuple (scaling factor, prefix), for example (1000, 'k') or (0.001, 'm'). The values
-    should be divided by the first element, and the unit label prepended with the second element.
-    If the values are zero the scaling factor is 1.
-    """
-    prefixes = {
-        -24: "y",
-        -21: "z",
-        -18: "a",
-        -15: "f",
-        -12: "p",
-        -9: "n",
-        -6: "\N{MICRO SIGN}",
-        -3: "m",
-        0: "",
-        3: "k",
-        6: "M",
-        9: "G",
-        12: "T",
-        15: "P",
-        18: "E",
-        21: "Z",
-        24: "Y",
-    }
-    # We apply a simple algorithm: get the element with largest magnitude, then map according to
-    # [0.001, 1) -> 0.001x/milli,
-    # [1, 1000) -> no scaling,
-    # [1000, 1e6) -> 1000x/kilo,
-    # and so on.
-    max_value = max(np.abs(values))
-    exponent = float(
-        3
-        * np.floor_divide(np.log10(max_value, out=np.zeros(1), where=max_value > 0), 3)
-    )
-    # Clip the scaling to the allowable range.
-    exponent_clipped = np.clip(exponent, -24, 24)
-    return 10 ** exponent_clipped, prefixes[exponent_clipped]
-
-
 def _create_plots_data_from_control(
     name, xdata, values, polar, unit_symbol, two_pi_factor
 ):
@@ -361,7 +328,7 @@ def _create_plots_data_from_control(
 
     if not np.iscomplexobj(values):
         # Real control.
-        prefix_scaling, prefix = _get_units(scaled_values)
+        prefix_scaling, prefix = get_units(scaled_values)
         return [
             _PlotData(
                 xdata=xdata,
@@ -372,7 +339,7 @@ def _create_plots_data_from_control(
 
     if polar:
         # Complex control, split into polar coordinates.
-        prefix_scaling, prefix = _get_units(scaled_values)
+        prefix_scaling, prefix = get_units(scaled_values)
         return [
             _PlotData(
                 xdata=xdata,
@@ -380,15 +347,13 @@ def _create_plots_data_from_control(
                 ylabel=f"{scaled_name}\nModulus\n({prefix}{unit_symbol})",
             ),
             _PlotData(
-                xdata=xdata,
-                values=np.angle(values),
-                ylabel=f"{name}\nAngle\n(rad)",
+                xdata=xdata, values=np.angle(values), ylabel=f"{name}\nAngle\n(rad)"
             ),
         ]
 
     # Complex control, split into rectangle coordinates.
-    prefix_scaling_x, prefix_x = _get_units(np.real(scaled_values))
-    prefix_scaling_y, prefix_y = _get_units(np.imag(scaled_values))
+    prefix_scaling_x, prefix_x = get_units(np.real(scaled_values))
+    prefix_scaling_y, prefix_y = get_units(np.imag(scaled_values))
     return [
         _PlotData(
             xdata=xdata,
@@ -425,13 +390,14 @@ def plot_sequences(figure, seq):
     plots_data: List[Any] = []
     for name, pulses in seq.items():
         offsets = [pulse["offset"] for pulse in pulses]
-        rotations = [pulse["rotation"] for pulse in pulses]
-        rotations = np.array(rotations)
+        rotations = np.array([pulse["rotation"] for pulse in pulses])
         plots_data = plots_data + _create_plots_data_from_sequence(
             name, offsets, rotations
         )
 
-    axes_list = _create_axes(figure, len(plots_data), width=10.0, height=2.5)
+    axes_list = create_axes(
+        figure, 1, len(plots_data), width=FIG_WIDTH, height=FIG_WIDTH / 4, share_x=True
+    ).flatten()
 
     for axes, plot_data in zip(axes_list, plots_data):
         # The plot_data.offsets array contains only the points where the
@@ -442,7 +408,7 @@ def plot_sequences(figure, seq):
         # function triples the number of points in the time array so that
         # the pulses can be drawn like that.
         times = np.repeat(plot_data.offsets, 3)
-        time_scaling, time_prefix = _get_units(times)
+        time_scaling, time_prefix = get_units(times)
         times /= time_scaling
 
         # Besides three points for each pulse, two extra points have to be
@@ -468,7 +434,7 @@ def plot_sequences(figure, seq):
 
         axes.set_ylabel(plot_data.label)
 
-    axes_list[-1].set_xlabel("Time ({0}s)".format(time_prefix))
+    axes_list[-1].set_xlabel(f"Time ({time_prefix}s)")
 
 
 _PlotSeqData = namedtuple("_PlotSeqData", ["label", "offsets", "rotations"])
@@ -483,57 +449,18 @@ def _create_plots_data_from_sequence(name, offsets, rotations):
             _PlotSeqData(
                 offsets=offsets,
                 rotations=rotations,
-                label="{0}\nrotations\n(rad)".format(name),
+                label=f"{name}\nrotations\n(rad)",
             )
         ]
     return [
         _PlotSeqData(
             offsets=offsets,
             rotations=np.abs(rotations),
-            label="{0}\nrotations\n(rad)".format(name),
+            label=f"{name}\nrotations\n(rad)",
         ),
         _PlotSeqData(
             offsets=offsets,
             rotations=np.angle(rotations),
-            label="{0}\nazimuthal angles\n(rad)".format(name),
+            label=f"{name}\nazimuthal angles\n(rad)",
         ),
     ]
-
-
-def _create_axes(figure, count, width, height):
-    """
-    Creates a set of axes with default axis labels and axis formatting.
-
-    The axes are stacked vertically, and share an x axis (automatically labeled with 'Time (s)').
-
-    Parameters
-    ----------
-    figure : matplotlib.figure.Figure
-        The matplotlib Figure in which the axes should be placed. The dimensions of the Figure will
-        be overridden by this method.
-    count : int
-        The number of axes to create.
-    width : float
-        The width (in inches) for each axes.
-    height : float
-        The height (in inches) for each axes.
-
-    Returns
-    -------
-    list
-        The list of Axes objects.
-    """
-    figure.set_figheight(height * count)
-    figure.set_figwidth(width)
-    figure.subplots_adjust(hspace=0.2)
-
-    axes_list = figure.subplots(
-        nrows=count, ncols=1, sharex=True, sharey=False, squeeze=False
-    )[:, 0]
-
-    # Set axis formatting.
-    for axes in axes_list:
-        axes.yaxis.set_major_formatter(ScalarFormatter())
-        axes.xaxis.set_major_formatter(ScalarFormatter())
-
-    return axes_list
