@@ -1,4 +1,4 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
@@ -106,7 +106,7 @@ def plot_discriminator(
         raise ValueError(
             "Background can only be plotted for individual "
             "qubit discriminators. Qubit mask has length "
-            "%i != 1" % len(discriminator.handler.qubit_mask)
+            f"{len(discriminator.handler.qubit_mask)} != 1"
         )
 
     x_data = np.array(discriminator.handler.xdata)
@@ -149,13 +149,8 @@ def plot_discriminator(
                 data[exp_state]["Q"].extend(dat[:, n_qubits + q_idx])
 
             # Plot the data by expected state.
-            for exp_state in data:
-                ax.scatter(
-                    data[exp_state]["I"],
-                    data[exp_state]["Q"],
-                    label=exp_state,
-                    alpha=0.5,
-                )
+            for exp_state, value in data.items():
+                ax.scatter(value["I"], value["Q"], label=exp_state, alpha=0.5)
 
                 if flag_misclassified:
                     y_disc = np.array(
@@ -175,7 +170,7 @@ def plot_discriminator(
 
     if title:
         for idx, q in enumerate(qubits_to_plot):  # pylint: disable=invalid-name
-            axs[idx].set_title("Qubit %i" % q)
+            axs[idx].set_title(f"Qubit {q}")
 
     for ax in axs:  # pylint: disable=invalid-name
         ax.set_xlabel("I (arb. units)")
